from cpl_core.logging import LoggerABC
from discord.ext import commands

from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.configuration.bot_settings import BotSettings
from gismo_core.configuration.discord_settings import DiscordSettings
from modules_core.abc.module_service_abc import ModuleServiceABC


class BotService(BotServiceABC, commands.Bot):

    def __init__(self, logger: LoggerABC, modules: ModuleServiceABC, discord_settings: DiscordSettings, bot_settings: BotSettings):
        # services
        self._logger = logger
        self._modules = modules
        
        # settings
        self._discord_settings = discord_settings
        self._bot_settings: BotSettings = bot_settings
        
        # setup super
        commands.Bot.__init__(self, command_prefix=bot_settings.prefix, help_command=None)

    async def start_async(self):
        self._logger.trace(__name__, 'Try to connect to discord')
        self.run(self._discord_settings.token)
        # continue at on_ready
        
    async def on_ready(self):
        self._logger.info(__name__, 'Connected to discord')
        await self._modules.start_modules()

    async def stop_async(self):
        self._logger.debug(__name__, f'Try to stop {BotService}')
        try:
            pass
            # save data
        except Exception as e:
            self._logger.error(__name__, 'Stop failed', e)
            
