from os import path

import discord
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List
from discord.ext import commands
from modules_core.abc.module_abc import ModuleABC
from modules_core.abc.module_service_abc import ModuleServiceABC


class ModuleService(ModuleServiceABC):

    def __init__(self, logger: LoggerABC, services: ServiceProviderABC, env: ApplicationEnvironmentABC):
        self._logger = logger
        self._services = services
        self._env = env
        self._modules: List[ModuleABC] = List()
        self._modules.extend(ModuleABC.__subclasses__())

    async def start_modules(self):
        self._logger.trace(__name__, 'Start loading modules')
        modules = self._modules.where(lambda m: hasattr(m, 'on_ready') and callable(m.on_ready))
        for module_type in modules:
            module = self._services.get_service(module_type)
            await module.on_ready()
